#!/bin/sh -e

# Copyright (C) 2021, 2023 |Méso|Star> (contact@meso-star.com)
# Copyright (C) 2021 Centre National de le Recherche Scientifique
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

path="$(dirname "${0}")"

mesh="${path}/tetra_mesh.smsh"
therm_props="${path}/thermodynamic_conditions.atrtp"
refract_ids="${path}/refraction_index.atrri"

laser="${laser}:pos=-1.0,0.0,0.1"
laser="${laser}:tgt=1.0,0.0,0.1"
laser="${laser}:up=0,0,1"
laser="${laser}:sz=0.001,0.3"

laser_flux_density="1000"

image_def="240x360"
image_spp="256"
image="def=${image_def}:spp=${image_spp}"

camera="${camera}:pos=0.0,-0.04,0.017"
camera="${camera}:tgt=0.0,0.0,0.017"
camera="${camera}:up=0,0,1"
camera="${camera}:fov=70"

wavelength="532"

rdgfa="${rdgfa}:dimension=1.80"
rdgfa="${rdgfa}:prefactor=1.30"

octree_tau="1"
octree_def="1024"

force_overwrite="-f"
simd="-s"
verbose="-v"

cache="gulder_${octree_def}_T${octree_tau}$([ -n "${simd}" ] && echo "_simd").cache"
output="gulder_image_${image_def}x${image_spp}.txt"

set -x
htrdr-combustion "${force_overwrite}" "${simd}" "${verbose}"\
 -m "${mesh}"\
 -p "${therm_props}"\
 -r "${refract_ids}"\
 -l "${laser}"\
 -D "${laser_flux_density}"\
 -i "${image}"\
 -C "${camera}"\
 -w "${wavelength}"\
 -F "${rdgfa}"\
 -T "${octree_tau}"\
 -V "${octree_def}"\
 -O "${cache}"\
 -o "${output}"
