c     Copyright (C)
c     2021 Mésotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine distance_2points(dim,x1,x2,d)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the distance between two positions defined
c     by their cartesian coordinates
c
c     Input:
c       + dim: space dimension
c       + x1: first position
c       + x2: second position
c
c     Output:
c       + d: distance between the two positions
c
c     I/O
      integer dim
      double precision x1(1:Ndim_mx)
      double precision x2(1:Ndim_mx)
      double precision d
c     temp
      integer i
      double precision d2
c     label
      integer stlren
      character*(Nchar_mx) label
      label='subroutine distance_2points'

      d2=0.0D+0
      do i=1,dim
         d2=d2+(x2(i)-x1(i))**2.0D+0
      enddo
      d=dsqrt(d2)

      return
      end



      subroutine spher2cart(parent,verif,x_spherical,x_cartesian)
      implicit none
      include 'max.inc'
c     
c     Purpose: to convert (r,theta,phi) spherical coordinates
c     into (x,y,z) cartesian coordinates
c
c     Input:
c       + parent: name of the routine that is calling "spher2cart"
c       + verif: true if angles checking required; false otherwise
c       + x_spherical: spherical coordinates of the position: r (m), theta (rad) in [-pi/2,pi/2], phi (rad) in [0:2*pi]
c
c     Output:
c       + x_cartesian: cartesian coordinates of the position: x (m), y (m), z (m)
c
c     I/O
      character*(Nchar_mx) parent
      logical verif
      double precision x_spherical(1:Ndim_mx)
      double precision x_cartesian(1:Ndim_mx)
c     label
      character*(Nchar_mx) label
      label='subroutine spher2cart from: '//trim(parent)

      if (verif) then
         call test_r(x_spherical(1),label)
         call test_theta(x_spherical(2),label)
         call test_phi(x_spherical(3),label)
      endif

      x_cartesian(1)=x_spherical(1)
     &     *dcos(x_spherical(2))
     &     *dcos(x_spherical(3))
      x_cartesian(2)=x_spherical(1)
     &     *dcos(x_spherical(2))
     &     *dsin(x_spherical(3))
      x_cartesian(3)=x_spherical(1)
     &     *dsin(x_spherical(2))
      
      return
      end



      subroutine cart2spher(x_cartesian,x_spherical)
      implicit none
      include 'max.inc'
      include 'param.inc'
c     
c     Purpose: to convert (x,y,z) cartesian coordinates
c     into (r,theta,phi) spherical coordinates
c
c     Input:
c       + x_cartesian: cartesian coordinates of the position: x (m), y (m), z (m)
c
c     Output:
c       + x_spherical: spherical coordinates of the position: r (m), theta (rad) in [-pi/2,pi/2], phi (rad) in [0:2*pi]
c
c     I/O
      double precision x_cartesian(1:Ndim_mx)
      double precision x_spherical(1:Ndim_mx)
c     temp
      double precision cosp,sinp,cost,sint
      double precision a,t1,t2
c     label
      character*(Nchar_mx) label
      label='subroutine cart2spher'

      a=dsqrt(x_cartesian(1)**2.0D+0
     &     +x_cartesian(2)**2.0D+0)
      x_spherical(1)=dsqrt(x_cartesian(1)**2.0D+0
     &     +x_cartesian(2)**2.0D+0
     &     +x_cartesian(3)**2.0D+0)
      if (a.eq.0.0D+0) then
         cosp=1.0D+0
         sinp=0.0D+0
      else
         cosp=x_cartesian(1)/a
         sinp=x_cartesian(2)/a
      endif
      cost=a/x_spherical(1)
      sint=x_cartesian(3)/x_spherical(1)
      
      t1=dacos(dabs(cosp))
      if (x_cartesian(1).ge.0.0D+0) then
         if (sinp.ge.0.0D+0) then
            x_spherical(3)=t1
         else
            x_spherical(3)=2.0D+0*pi-t1
         endif
      else
         if (sinp.ge.0.0D+0) then
            x_spherical(3)=pi-t1
         else
            x_spherical(3)=pi+t1
         endif
      endif

      t2=dacos(cost)
      if (sint.ge.0.0D+0) then
         x_spherical(2)=t2
      else
         x_spherical(2)=-t2
      endif
      
      return
      end
