c     Copyright (C)
c     2021 Mésotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine volumic_fraction(dim,box_volume,x,fv)
      implicit none
      include 'max.inc'
      include 'com_properties.inc'
c
c     Purpose: to provide the soot volumic fraction field
c
c     Input:
c       + dim: dimension of space
c       + box_volume: volume limits of the surrounding box
c       + fv_max: maximum soot volumic fraction [m³/m³] (via 'com_properties.inc')
c       + x: position
c     
c     Output:
c       + fv: soot volumic fraction [m³ of soot / m³]
c     
c     I/O
      integer dim
      double precision box_volume(1:Ndim_mx,1:2)
      double precision x(1:Ndim_mx)
      double precision fv
c     temp
      double precision L
c     label
      character*(Nchar_mx) label
      label='subroutine volumic_fraction'

      L=box_volume(1,2)-box_volume(1,1)
      fv=fv_max
     &     *(0.5D+0-x(1)/L)
     &     *(1.0D+0-dsqrt(((x(2)/L-0.5D+0)**2.0D+0
     &     +(x(3)/L-0.5D+0)**2.0D+0)
     &     /0.50D+0))
      
      return
      end



      subroutine absorption_coefficient(dim,box_volume,x,ka)
      implicit none
      include 'max.inc'
      include 'com_properties.inc'
      include 'rdgfa.inc'
c
c     Purpose: to provide the absorption coefficient field
c
c     Input:
c       + dim: dimension of space
c       + box_volume: volume limits of the surrounding box
c       + x: position
c     
c     Output:
c       + ka: absorption coefficient [m⁻¹]
c     
c     I/O
      integer dim
      double precision box_volume(1:Ndim_mx,1:2)
      double precision x(1:Ndim_mx)
      double precision ka
c     temp
      double precision L
      double precision fv,rho
c     label
      character*(Nchar_mx) label
      label='subroutine absorption_coefficient'

      L=box_volume(1,2)-box_volume(1,1)
      call volumic_fraction(dim,box_volume,x,fv)
      rho=fv/Va                 ! [nm³ of soot / nm³]/[nm³ of soot/aggregate]=[aggregates/nm³]
      
      ka=1.0D9*rho*Cabs         ! [m⁻¹]
      
      return
      end


      
      subroutine scattering_coefficient(dim,box_volume,x,ks)
      implicit none
      include 'max.inc'
      include 'com_properties.inc'
      include 'rdgfa.inc'
c
c     Purpose: to provide the scattering coefficient field
c
c     Input:
c       + dim: dimension of space
c       + box_volume: volume limits of the surrounding box
c       + x: position
c     
c     Output:
c       + ks: scattering coefficient [m⁻¹]
c     
c     I/O
      integer dim
      double precision box_volume(1:Ndim_mx,1:2)
      double precision x(1:Ndim_mx)
      double precision ks
c     temp
      double precision L
      double precision fv,rho
c     label
      character*(Nchar_mx) label
      label='subroutine scattering_coefficient'

      L=box_volume(1,2)-box_volume(1,1)
      call volumic_fraction(dim,box_volume,x,fv)
      rho=fv/Va                 ! [nm³ of soot / nm³]/[nm³ of soot/aggregate]=[aggregates/nm³]

      ks=1.0D9*rho*Csca         ! [m⁻¹]
      
      return
      end
      

      
      subroutine extinction_coefficient(dim,box_volume,x,kext)
      implicit none
      include 'max.inc'
c
c     Purpose: to provide the extinction coefficient field
c
c     Input:
c       + dim: dimension of space
c       + box_volume: volume limits of the surrounding box
c       + x: position
c     
c     Output:
c       + kext: extinction coefficient [m⁻¹]
c     
c     I/O
      integer dim
      double precision box_volume(1:Ndim_mx,1:2)
      double precision x(1:Ndim_mx)
      double precision kext
c     temp
      double precision ka,ks
c     label
      character*(Nchar_mx) label
      label='subroutine extinction_coefficient'

      call absorption_coefficient(dim,box_volume,x,ka)
      call scattering_coefficient(dim,box_volume,x,ks)
      kext=ka+ks

      return
      end
